"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const DependencyManager_1 = require("./DependencyManager");
const aws_sdk_1 = require("aws-sdk");
const Resources_1 = require("./Resources");
class Client {
    constructor(options) {
        this.accountID = null;
        this.options = options;
        this.dependencyManager = new DependencyManager_1.DependencyManager();
        this.sqsClient = new aws_sdk_1.SQS();
        this.snsClient = new aws_sdk_1.SNS();
    }
    isMockClient() {
        return this.options.mockClient;
    }
    async publish(publishCommand) {
        // Create resources
        await this.dependencyManager.satisfySNSPublishDependencies(this, publishCommand);
        let accountID = await this.getAccountID();
        let topicARN = Resources_1.generateARN('sns', publishCommand.topic.region, accountID, publishCommand.topic.topicName);
        console.log("Sending SNS Message");
        await this.snsClient.publish({
            Message: publishCommand.message.stringContent,
            TopicArn: topicARN,
            MessageAttributes: publishCommand.message.attributes
        }).promise();
    }
    async getAccountID() {
        // Lazy load grab accountID
        if (this.accountID) {
            return this.accountID;
        }
        else {
            const sts = new aws_sdk_1.STS();
            let ret = await sts.getCallerIdentity({}).promise();
            if (ret.Account) {
                this.accountID = ret.Account;
                return ret.Account;
            }
            else {
                throw new Error('Account Load Failed');
            }
        }
    }
}
exports.Client = Client;
