"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DependencyManager = void 0;
const DeferredPromise_1 = require("./DeferredPromise");
class DependencyManager {
    constructor() {
        // Store whether each dependency identifier is currently processing
        this.dependencyProcessing = {};
        // Store all promises waiting for a dependency to finish completion
        this.dependencySatisfactionPromises = {};
        // Store all cached values of whether each dependency identifier is already satisfied
        this.dependencySatisfiedCache = {};
    }
    satisfySNSPublishDependencies(client, publishCommand) {
        let dependencyIdentifier = publishCommand.dependencyIdentifier;
        let dependencies = null;
        // If custom identifier is not set, we cannot use optimisation, and recalculate each time
        if (dependencyIdentifier == null) {
            dependencies = publishCommand.getDependencies();
            dependencyIdentifier = "";
            for (let i = 0; i < dependencies.length; i++) {
                dependencyIdentifier += dependencies[i].toString() + "_";
            }
        }
        if (dependencyIdentifier in this.dependencySatisfiedCache) {
            return Promise.resolve();
        }
        else {
            if (dependencyIdentifier in this.dependencyProcessing) {
                // This dependency is being fulfilled currently. Just wait
                let thisPromise = new DeferredPromise_1.DeferredPromise();
                this.dependencySatisfactionPromises[dependencyIdentifier].push(thisPromise);
                return thisPromise.promise;
            }
            else {
                // Not processing and not in cache. This is a fresh case
                if (dependencies == null) {
                    dependencies = publishCommand.getDependencies();
                }
                this.dependencyProcessing[dependencyIdentifier] = true;
                let thisPromise = new DeferredPromise_1.DeferredPromise();
                this.dependencySatisfactionPromises[dependencyIdentifier] = [thisPromise];
                this.satisfyDependencies(client, dependencyIdentifier, dependencies);
                return thisPromise.promise;
            }
        }
    }
    async satisfyDependencies(client, identifier, resources) {
        // Ensure and create each resource sequentially
        for (let i = 0; i < resources.length; i++) {
            await resources[i].ensureAndCreate(client);
        }
        // Satisfy all waiting promises
        let deferredPromises = this.dependencySatisfactionPromises[identifier];
        for (let i = 0; i < deferredPromises.length; i++) {
            deferredPromises[i].resolve();
        }
        this.dependencySatisfactionPromises[identifier] = [];
        // Register that this dependency is now satisfied
        this.dependencySatisfiedCache[identifier] = true;
    }
}
exports.DependencyManager = DependencyManager;
